/* dump.c -- primitive creation steps to make a PostScript file
 * Copyright (C) 1994 David Deaven
 *
 * This file is part of xyplot.
 *
 * xyplot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * xyplot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with xyplot; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: dump.c,v 1.3 1995/10/10 15:03:39 deaven Exp $
 */
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "dmd/memory.h"
#include "xyplot.h"

extern double xmin, xmax, ymin, ymax;
extern int log_x, log_y;
extern int breakflag;

/* Make PostScript plot instructions using the current
 * settings to a file.
 */
void DumpPlot(FILE *fout, XYFileInfo *fi, char c)
{
  double x,y,xerr,yerr;
  double xx,yy;
  double lastx=0.0, lasty=0.0;
  int out=0, lastout=0;
  while(XYReadStream(fi,&x,&y,&xerr,&yerr)!=EOF)
  {
    if(log_x)x=log(fabs((double) x));
    if(log_y)y=log(fabs((double) y));
    out = (x < xmin || x > xmax || y < ymin || y > ymax);
    switch(c)
    {
      case 'c':
	if(breakflag)
	{
	  fprintf(fout, "stroke startline\n");
	  breakflag = 0;
	}
	if(out && !lastout)
	{
	  xx=x; yy=y;
	  fixpoint(&xx, &yy, (y - lasty) / (x - lastx));
	  fprintf(fout, "%.6g %.6g %c\n",xx,yy,c);
	}
	else if(!out && lastout)
	{
	  xx=lastx; yy=lasty;
	  fixpoint(&xx, &yy, (y - lasty) / (x - lastx));
	  fprintf(fout, "%.6g %.6g %c\n",xx,yy,c);
	  fprintf(fout, "%.6g %.6g %c\n",x,y,c);
	}
	else if(!out)
	  fprintf(fout, "%.6g %.6g %c\n",x,y,c);
	break;
      default:
	if(!out)
	{
	  fprintf(fout, "%.6g %.6g %c",x,y,c);
	  if(c!='c') {
	    if(fi->xerr>0 && xerr!=0.0)fprintf(fout, " %.6g xbar rp",xerr);
	    if(fi->yerr>0 && yerr!=0.0)fprintf(fout, " %.6g ybar rp",yerr);
	  }
	  fprintf(fout, "\n");
	}
	break;
    } /* switch */
    lastout = out;
    lastx = x;
    lasty = y;
  }
}

/* Print the data as ordinary numbers (no error bars), with full
 * machine precision.
 */
void DumpData(FILE *f, XYFileInfo *fi)
{
  double x,y,xerr,yerr;
  while(XYReadStream(fi,&x,&y,&xerr,&yerr)!=EOF) {
    if(log_x)x=log(fabs((double) x));
    if(log_y)y=log(fabs((double) y));
    fprintf(f, "%.14g %.14g\n",x,y);
  }
}

/* Make UNIX plot instructions */
void DumpUnix(FILE *f, XYFileInfo *fi)
{
  double x,y,xerr,yerr;
  double dx,dy;
  int n=0,c=fi->connect;
  dx=(xmax-xmin)/500.0;
  dy=(ymax-ymin)/500.0;
  while(XYReadStream(fi,&x,&y,&xerr,&yerr)!=EOF) {
    if(log_x)x=log(fabs((double) x));
    if(log_y)y=log(fabs((double) y));
    ++n;
    if(n==1 && c) {
      fputc('m',f); putUnix(f,x,y);
    } else {
      if(c) {
	fputc('n',f); putUnix(f,x,y); 
      } else {
	fputc('l',f); putUnix(f,x-dx,y-dy); putUnix(f,x-dx,y+dy);
	fputc('n',f); putUnix(f,x+dx,y+dy);
	fputc('n',f); putUnix(f,x+dx,y-dy);
	fputc('n',f); putUnix(f,x-dx,y-dy);
      }
    }
  }
}

