/* dvips2xy -- scan dvips PostScript(tm) and output
 *           xyplot-compatible PostScript(tm).
 * Copyright (C) 1995 David Deaven
 *
 * This file is part of xyplot.
 *
 * xyplot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * xyplot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with xyplot; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * $Id: dvips2xy.c,v 1.3 1995/09/19 20:06:59 deaven Exp $
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define MAXSTRING 512  /* maximum length of strings */

static char *usage="usage: %s [-o ps-file] dvips-file\n";

static char line[MAXSTRING];

enum modes {HEADER, SETUP, LABEL, TRAILER};

void main(int argc, char **argv)
{
  int arg, N = 0;
  char *s;
  enum modes mode;
  char *ifn = "-", *ofn = "-";
  FILE *fin = stdin, *fout = stdout;

  /* command line options.
   */

  for(arg=1; arg<argc; ++arg) {
    if(argv[arg][0]=='-') {
      switch(argv[arg][1]) {
	case 'o':
	  ofn = argv[++arg];
	  break;

	default:
	  fprintf(stderr, usage, argv[0]);
	  exit(1);
      }
    }
    else
      ifn = argv[arg];
  }

  fin = (strcmp(ifn, "-") ? fopen(ifn, "r") : stdin);
  if(fin == NULL) {
    fprintf(stderr, "%s: cannot open `%s'\n", argv[0], ifn);
    exit(1);
  }
  fout = (strcmp(ofn, "-") ? fopen(ofn, "w") : stdout);
  if(fout == NULL) {
    fprintf(stderr, "%s: cannot open `%s'\n", argv[0], ofn);
    exit(1);
  }

  /* Scan the output, removing structured comments and looking for 
   * page definitions -- each page becomes a separate definition.
   */

  mode = HEADER;
  fprintf(fout, "%%------------- dvips2xy begin --------------\n");
  while(fgets(line, MAXSTRING, fin) != NULL) {

    for(s=line; isspace(*s); ++s);
    if(*s == '%') { /* special line, might have information */

      if(s[1] == '%' && mode == LABEL) {
	fprintf(fout, "grestore end} def\n");
	mode = HEADER;
      }

      if(strstr(s, "%%Page:") != NULL) { /* page def */
	mode = LABEL;
	fprintf(fout, "/label_%d {TeXDict begin gsave"
	  " currentpoint translate 72 600 div dup neg scale\n", ++N);
      }
      else if(strstr(s, "%%Trailer") != NULL)
	mode = TRAILER;
      else if(strstr(s, "%%BeginSetup") != NULL) {
	fprintf(fout, "TeXDict begin /bop {pop pop} def /eop {} def end\n");
	mode = SETUP;
      }

    }
    else { /* normal non-comment line */

      switch(mode) {
	case HEADER:
	case LABEL:
	  fputs(line, fout);
	  break;

	case SETUP:
	case TRAILER:
	  break;
      }
    }
  }
  fprintf(fout, "%%------------- dvips2xy end ----------------\n");

  if(fin != stdin) fclose(fin);
  if(fout != stdout) fclose(fout);

  exit(0);
}

